import pytest

from django import forms
from django.template import Context, Template
from django.utils.translation import activate, deactivate
from django.utils.translation import gettext as _

from crispy_forms.bootstrap import InlineRadios  # # TODO switch to bulma specific
from crispy_forms.bootstrap import (
    Alert,
    AppendedText,
    FieldWithButtons,
    InlineCheckboxes,
    InlineField,
    PrependedAppendedText,
    PrependedText,
    StrictButton,
    Tab,
    TabHolder,
)
from crispy_forms.helper import FormHelper
from crispy_forms.layout import HTML, Field, Layout, MultiWidgetField
from crispy_forms.utils import render_crispy_form

from .forms import (
    CheckboxesSampleForm,
    CustomCheckboxSelectMultiple,
    CustomRadioSelect,
    GroupedChoiceForm,
    InputsForm,
    SampleForm,
    SampleFormCustomWidgets,
)
from .utils import parse_expected, parse_form


@pytest.mark.skip(reason="bulma")
def test_email_field():
    form = SampleForm()
    form.helper = FormHelper()
    form.helper.layout = Layout("email")
    assert parse_form(form) == parse_expected("test_email_field.html")


def test_field_with_custom_template():
    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        Field("email", template="custom_field_template.html")
    )

    html = render_crispy_form(test_form)
    assert "<h1>Special custom field</h1>" in html


def test_multiwidget_field():
    template = Template(
        """
        {% load crispy_forms_tags %}
        {% crispy form %}
    """
    )

    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        MultiWidgetField(
            "datetime_field",
            attrs=(
                {"rel": "test_dateinput"},
                {"rel": "test_timeinput", "style": "width: 30px;", "type": "hidden"},
            ),
        )
    )

    c = Context({"form": test_form})

    html = template.render(c)

    assert html.count('class="dateinput') == 1
    assert html.count('rel="test_dateinput"') == 1
    assert html.count('rel="test_timeinput"') == 2
    assert html.count('style="width: 30px;"') == 2
    assert html.count('type="hidden"') == 2


def test_field_type_hidden():
    template = Template(
        """
        {% load crispy_forms_tags %}
        {% crispy test_form %}
    """
    )

    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        Field("email", type="hidden", data_test=12),
        Field("datetime_field"),
    )

    c = Context({"test_form": test_form})
    html = template.render(c)

    # Check form parameters
    assert html.count('data-test="12"') == 1
    assert html.count('name="email"') == 1
    assert html.count('class="dateinput') == 1
    assert html.count('class="timeinput') == 1


def test_field_wrapper_class(settings):
    form = SampleForm()
    form.helper = FormHelper()
    form.helper.layout = Layout(Field("email", wrapper_class="testing"))

    html = render_crispy_form(form)
    assert html.count('class="field testing"') == 1


def test_html_with_carriage_returns(settings):
    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        HTML(
            """
            if (a==b){
                // some comment
                a+1;
                foo();
            }
        """
        )
    )
    html = render_crispy_form(test_form)
    assert html.count("\n") == 24


def test_i18n():
    activate("es")
    form = SampleForm()
    form.helper = FormHelper()
    form.helper.layout = Layout(HTML(_("Enter a valid value.")))
    html = render_crispy_form(form)
    assert "Introduzca un valor válido" in html

    deactivate()


def test_remove_labels():
    form = SampleForm()
    # remove boolean field as label is still printed in bulma
    del form.fields["is_company"]

    for fields in form:
        fields.label = False

    html = render_crispy_form(form)

    assert "<label" not in html


@pytest.mark.parametrize(
    "input,expected",
    [
        ("text_input", "text_input.html"),
        ("text_area", "text_area.html"),
        # ("checkboxes", "checkboxes.html"),
        # ("radio", "radio.html"),
        ("single_checkbox", "single_checkbox.html"),
        ("select_input", "test_select.html"),
    ],
)
def test_inputs(input, expected):
    form = InputsForm()
    form.helper = FormHelper()
    form.helper.layout = Layout(input)
    assert parse_form(form) == parse_expected(expected)


@pytest.mark.skip(reason="InlineRadios")
def test_custom_django_widget():
    # Make sure an inherited RadioSelect gets rendered as it
    form = SampleFormCustomWidgets()
    assert isinstance(form.fields["inline_radios"].widget, CustomRadioSelect)
    form.helper = FormHelper()
    form.helper.layout = Layout("inline_radios")

    html = render_crispy_form(form)
    assert 'class="form-check-input"' in html

    # Make sure an inherited CheckboxSelectMultiple gets rendered as it
    assert isinstance(form.fields["checkboxes"].widget, CustomCheckboxSelectMultiple)
    form.helper.layout = Layout("checkboxes")
    html = render_crispy_form(form)
    assert 'class="form-check-input"' in html


@pytest.mark.skip(reason="prepended_appended_text")
def test_prepended_appended_text():
    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        PrependedAppendedText(
            "email", "@<>&", "gmail.com", css_class="form-control-lg"
        ),
        AppendedText("password1", "#"),
        PrependedText("password2", "$"),
    )
    assert parse_form(test_form) == parse_expected("test_prepended_appended_text.html")


@pytest.mark.skip(reason="InlineRadios")
def test_inline_radios():
    test_form = CheckboxesSampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(InlineRadios("inline_radios"))
    html = render_crispy_form(test_form)
    assert html.count('form-check-inline"') == 2


@pytest.mark.skip(reason="bootstrap")
def test_alert():
    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.form_tag = False
    test_form.helper.layout = Layout(
        Alert(content="Testing...", css_class="alert-primary"),
        Alert(content="Testing...", css_class="alert-primary", dismiss=False),
    )
    assert parse_form(test_form) == parse_expected("alert.html")


@pytest.mark.skip(reason="bootstrap")
def test_tab_and_tab_holder():
    test_form = SampleForm()
    test_form.helper = FormHelper()
    test_form.helper.layout = Layout(
        TabHolder(
            Tab(
                "one",
                "first_name",
                css_id="custom-name",
                css_class="first-tab-class active",
            ),
            Tab("two", "password1", "password2"),
        )
    )
    html = render_crispy_form(test_form)

    assert (
        html.count(
            '<ul class="nav nav-tabs"> <li class="nav-item">'
            '<a class="nav-link active" href="#custom-name" data-bs-toggle="tab">'
            "One</a></li>"
        )
        == 1
    )
    assert html.count("tab-pane") == 2

    assert html.count('class="tab-pane first-tab-class active"') == 1

    assert html.count('<div id="custom-name"') == 1
    assert html.count('<div id="two"') == 1
    assert html.count('name="first_name"') == 1
    assert html.count('name="password1"') == 1
    assert html.count('name="password2"') == 1


@pytest.mark.skip(reason="bootstrap")
def test_tab_helper_reuse():
    # this is a proper form, according to the docs.
    # note that the helper is a class property here,
    # shared between all instances
    class SampleForm(forms.Form):
        val1 = forms.CharField(required=False)
        val2 = forms.CharField(required=True)
        helper = FormHelper()
        helper.layout = Layout(
            TabHolder(
                Tab("one", "val1"),
                Tab("two", "val2"),
            )
        )

    # first render of form => everything is fine
    test_form = SampleForm()
    html = render_crispy_form(test_form)

    # second render of form => first tab should be active,
    # but not duplicate class
    test_form = SampleForm()
    html = render_crispy_form(test_form)
    assert html.count('class="nav-item active active"') == 0

    # render a new form, now with errors
    test_form = SampleForm(data={"val1": "foo"})
    html = render_crispy_form(test_form)
    tab_class = "tab-pane"
    # if settings.CRISPY_TEMPLATE_PACK == 'bootstrap4':
    # tab_class = 'nav-link'
    # else:
    # tab_class = 'tab-pane'
    # tab 1 should not be active
    assert html.count('<div id="one" \n    class="{} active'.format(tab_class)) == 0
    # tab 2 should be active
    assert html.count('<div id="two" \n    class="{} active'.format(tab_class)) == 1


@pytest.mark.skip(reason="InlineRadios")
def test_radio_attrs():
    form = CheckboxesSampleForm()
    form.fields["inline_radios"].widget.attrs = {"class": "first"}
    form.fields["checkboxes"].widget.attrs = {"class": "second"}
    html = render_crispy_form(form)
    assert 'class="first"' in html
    assert 'class="second"' in html


@pytest.mark.skip(reason="button")
def test_field_with_buttons():
    form = SampleForm()
    form.helper = FormHelper()
    form.helper.layout = Layout(
        FieldWithButtons(
            Field("password1", css_class="span4"),
            StrictButton("Go!", css_id="go-button"),
            StrictButton("No!", css_class="extra"),
            StrictButton("Test", type="submit", name="whatever", value="something"),
            css_class="extra",
            autocomplete="off",
        )
    )
    html = render_crispy_form(form)

    form_group_class = "mb-3"

    assert html.count('class="%s extra"' % form_group_class) == 1
    assert html.count('autocomplete="off"') == 1
    assert html.count('class="span4') == 1
    assert html.count('id="go-button"') == 1
    assert html.count("Go!") == 1
    assert html.count("No!") == 1
    assert html.count('class="btn"') == 2
    assert html.count('class="btn extra"') == 1
    assert html.count('type="submit"') == 1
    assert html.count('name="whatever"') == 1
    assert html.count('value="something"') == 1


@pytest.mark.skip(reason="prepended_appended_text")
def test_hidden_fields():
    form = SampleForm()
    # All fields hidden
    for field in form.fields:
        form.fields[field].widget = forms.HiddenInput()

    form.helper = FormHelper()
    form.helper.layout = Layout(
        AppendedText("password1", "foo"),
        PrependedText("password2", "bar"),
        PrependedAppendedText("email", "bar"),
        InlineCheckboxes("first_name"),
        InlineRadios("last_name"),
    )
    html = render_crispy_form(form)
    assert html.count("<input") == 5
    assert html.count('type="hidden"') == 5
    assert html.count("<label") == 0


@pytest.mark.skip(reason="InlineCheckboxes")
def test_multiplecheckboxes():
    test_form = CheckboxesSampleForm()
    html = render_crispy_form(test_form)
    assert html.count("checked") == 6

    test_form.helper = FormHelper(test_form)
    test_form.helper[1].wrap(InlineCheckboxes, inline=True)
    html = render_crispy_form(test_form)
    # TODO Fix this test
    # assert html.count('form-check-input"') == 3


@pytest.mark.skip(reason="InlineCheckboxes")
def test_multiple_checkboxes_unique_ids():
    test_form = CheckboxesSampleForm()
    html = render_crispy_form(test_form)

    expected_ids = [
        "checkboxes_0",
        "checkboxes_1",
        "checkboxes_2",
        "alphacheckboxes_0",
        "alphacheckboxes_1",
        "alphacheckboxes_2",
        "numeric_multiple_checkboxes_0",
        "numeric_multiple_checkboxes_1",
        "numeric_multiple_checkboxes_2",
    ]
    for id_suffix in expected_ids:
        expected_str = f'id="id_{id_suffix}"'
        assert html.count(expected_str) == 1


@pytest.mark.skip(reason="bootstrap")
def test_inline_field():
    form = SampleForm()
    form.helper = FormHelper()
    form.helper.layout = Layout(
        InlineField("first_name", wrapper_class="col-4"),
        InlineField("is_company", wrapper_class="col-4"),
    )
    form.helper.form_class = "row row-cols-lg-auto align-items-center"
    assert parse_form(form) == parse_expected("test_inline_field.html")


@pytest.mark.skip(reason="bootstrap")
def test_grouped_checkboxes_radios():
    form = GroupedChoiceForm()
    form.helper = FormHelper()
    form.helper.layout = Layout("checkbox_select_multiple")
    assert parse_form(form) == parse_expected("test_grouped_checkboxes.html")
    form.helper.layout = Layout("radio")
    assert parse_form(form) == parse_expected("test_grouped_radios.html")

    form = GroupedChoiceForm({})
    form.helper = FormHelper()
    form.helper.layout = Layout("checkbox_select_multiple")
    assert parse_form(form) == parse_expected("test_grouped_checkboxes_failing.html")
    form.helper.layout = Layout("radio")
    assert parse_form(form) == parse_expected("test_grouped_radios_failing.html")



#!/usr/bin/env python

from setuptools import setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()


setup(
    name="crispy-bulma",
    version="0.3.1",
    description="Bulma template pack for django-crispy-forms",
    long_description=readme + "\n\n" + history,
    author="Christoph Krybus",
    author_email="chris@ckrybus.com",
    keywords=["forms", "django", "crispy", "bulma"],
    url="https://github.com/ckrybus/crispy-bulma",
    license="MIT license",
    packages=["crispy_bulma"],
    install_requires=["Django>=2.2", "django-crispy-forms>=1.9.0"],
    extras_require={"test": ["pytest", "pytest-django"]},
    tests_require=["crispy-bulma[test]"],
    python_requires=">=3.6",
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from crispy_forms.layout import (
    HTML,
    BaseInput,
    ButtonHolder,
    Div,
    Field,
    Fieldset,
    Hidden,
    Layout,
    MultiField,
    MultiWidgetField,
)
from crispy_forms.utils import TEMPLATE_PACK, render_field

__all__ = [
    # Defined in this file
    "Button",
    "Column",
    "IconField",
    "Reset",
    "Row",
    "Submit",
    "UploadField",
    # Imported from CrispyForms itself
    "ButtonHolder",
    "Div",
    "Field",
    "Fieldset",
    "Hidden",
    "HTML",
    "Layout",
    "MultiField",
    "MultiWidgetField",
]


class UploadField(Field):
    def __init__(self, *args, **kwargs):
        if "css_class" in kwargs:
            kwargs["css_class"] += " file-input"
        else:
            kwargs["css_class"] = "file-input"

        super().__init__(*args, **kwargs)


class Submit(BaseInput):
    """
    Used to create a Submit button descriptor for the {% crispy %} template tag.
    >>> submit = Submit("Search the Site", "search this site")

    The first argument is also slugified and turned into the id for the submit button.
    """

    field_classes = "button is-primary"
    input_type = "submit"


class Button(BaseInput):
    """
    Used to create a Submit input descriptor for the {% crispy %} template tag.
    >>> button = Button("Button 1", "Press Me!")

    The first argument is also slugified and turned into the id for the button.
    """

    field_classes = "button"
    input_type = "button"


class Reset(BaseInput):
    """
    Used to create a Reset button input descriptor for the {% crispy %} template tag.
    >>> reset = Reset("Reset This Form", "Revert Me!")

    The first argument is also slugified and turned into the id for the button.
    """

    field_classes = "button is-text"
    input_type = "reset"


class Row(Div):
    """
    Layout object. It wraps fields in a div whose default class is "columns".
    >>> Row("form_field_1", "form_field_2", "form_field_3")
    """

    css_class = "columns"


class Column(Div):
    """
    Layout object. It wraps fields in a div whose default class is "column".

    >>> Column("form_field_1", "form_field_2")
    """

    css_class = "column"


class IconField(Field):
    template = "%s/layout/input_with_icon.html"

    def __init__(self, field, icon_prepend=None, icon_append=None, *args, **kwargs):
        self.field = field
        self.icon_prepend = icon_prepend
        self.icon_append = icon_append
        super().__init__(*args, **kwargs)

    def render(
        self,
        form,
        form_style,
        context,
        template_pack=TEMPLATE_PACK,
        extra_context=None,
        **kwargs
    ):
        extra_context = extra_context.copy() if extra_context is not None else {}
        extra_context.update(
            {
                "icon_prepend": self.icon_prepend,
                "icon_append": self.icon_append,
            }
        )
        template = self.get_template_name(template_pack)

        return render_field(
            self.field,
            form,
            form_style,
            context,
            template=template,
            template_pack=template_pack,
            extra_context=extra_context,
            **kwargs
        )
